function [psi,tprime,theta,phi,cs]=edgemorlet(sc,cj,sr,ki,cf,aa,rn,ot,nt,skip)
%
% EDGEMORLET -- renormalized Morlet wavelet with variable sample rate.
%
%     PSI = EDGEMORLET
%     calculates the Morlet mother wavelet with norm 2 and length 2*6+1.
%
%     PSI = EDGEMORLET(SC) 
%     calculates the renormalized Morlet wavelet at scale SC for |SC|>=1
%     with norm 2/SC^2 and length 2*floor(|SC|*6)+1.  Warning for |SC|<2.
%
%     PSI = EDGEMORLET(SC,CJ) 
%     calculates the renormalized Morlet wavelet at scale SC for CJ=1 
%     and its conjugate for CJ=-1.
%
%     PSI = EDGEMORLET(SC,CJ,SR) 
%     calculates the renormalized Morlet wavelet at sample scale SC*SR 
%     where integer SR>=1 defines the unit period for the scale axis in
%     terms of the sample rate.
%
%     PSI = EDGEMORLET(SC,CJ,SR,KI) 
%     calculates the renormalized Morlet wavelet at sample scale SC*SR
%     with length 2*floor(|SC*SR|*KI)+1 for KI>=4. Default KI=6.
%
%     PSI = EDGEMORLET(SC,CJ,SR,KI,CF) 
%     calculates the renormalized Morlet wavelet at sample scale SC*SR
%     with central frequency CF ~ 2*PI+1/4/PI.
%
%     PSI = EDGEMORLET(SC,CJ,SR,KI,CF,AA) 
%     calculates the renormalized Morlet wavelet at sample scale SC*SR
%     with mean(PSI)=0 for AA=1.  Default AA=0 neglects admissibility.
%
%     PSI = EDGEMORLET(SC,CJ,SR,KI,CF,AA,RN) 
%     calculates the unrenormalized Morlet wavelet at sample scale SC*SR
%     for RN=0.  Default RN=1 enforces renormalization.
%
%     PSI = EDGEMORLET(SC,CJ,SR,KI,CF,AA,RN,OT) 
%     calculates the renormalized Morlet wavelet at sample scale SC*SR
%     at offset OT for |OT|<=floor(|SC*SR|*KI).
%
%     PSI = EDGEMORLET(SC,CJ,SR,KI,CF,AA,RN,OT,NT) 
%     calculates the renormalized Morlet wavelet at sample scale SC*SR
%     at offset OT and data length NT in terms of the sample rate.
%
%     PSI = EDGEMORLET(SC,CJ,SR,KI,CF,AA,RN,OT,NT,SKIP) 
%     skips all sanity checks for SKIP='skip' -- not recommended.
%
%     [PSI,TPRIME] = EDGEMORLET(...) 
%     returns the wavelet in PSI and its duration in TPRIME.
%
%     [PSI,TPRIME,THETA] = EDGEMORLET(...) 
%     returns the wavelet in PSI, its duration in TPRIME, and the wave
%     in THETA.
%
%     [PSI,TPRIME,THETA,PHI] = EDGEMORLET(...) 
%     returns the wavelet in PSI, its duration in TPRIME, the wave in
%     THETA, and the window in PHI.
%
%     [PSI,TPRIME,THETA,PHI,CS] = EDGEMORLET(...) 
%     returns the wavelet in PSI, its duration in TPRIME, the wave in
%     THETA, the window in PHI, and the normalization constant in CS.
%
%     NOTE: Any input parameter may be replaced by [] to accept default
%     value. Eg, EDGEMORLET([],[],[],KI) calculates the mother wavelet
%     with resolution width KI and length 2*floor(KI)+1
%
%     See also: RCWTM, IRCWTM, XYCWTM, SHOWRCWT, AWDEMO
%
%     AlphaWavelet Toolbox Version 1.0.1
%     Copyright (C) 2010 Alphawave Research, robjohnson@alphawaveresearch.com
%     Released under GNU GPLv3 -- see Contents.m for more info.
%

%     This file is part of AlphaWavelet.
%
%     AlphaWavelet is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or (at
%     your option) any later version.
%
%     AlphaWavelet is distributed in the hope that it will be useful, but
%     WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%     General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see `http://www.gnu.org/licenses/'.
%
%     **************************************************************************

% Preliminaries...
if nargin>10, error('nargin > 10, try help edgemorlet'); end
if ~((nargin>9)&&(strcmpi(skip,'skip')))	% skip all checks if 10th argument is 'skip'
% Initialization...
if nargin<1 || isempty(sc), sc=1; end		% default scale 1
if nargin<2 || isempty(cj), cj=1; end		% default not conjugate
if nargin<3 || isempty(sr), sr=1; end		% default sample rate
if nargin<4 || isempty(ki), ki=6; end		% default resolution width
if nargin<5 || isempty(cf), cf=2*pi+1/4/pi; end	% default central frequency
if nargin<6 || isempty(aa), aa=0; end		% default admissibility off
if nargin<7 || isempty(rn), rn=1; end		% default renormalization on
if nargin<8 || isempty(ot), ot=0; end		% default offset 0
% Sanity check...
if nargin>8, if rem(nt,1)|(nt<=0), error('NT not integer or negative'); end
if nt<(2*floor(sr*ki)+1), error('NT less than mother wavelet'); end, else nt=0; end
if rem(ot,1), error('OT must be integer'); end
if ~((rn==1)|(rn==0)), error('RN must be 1 or 0'); end
if ~((aa==1)|(aa==0)), error('AA must be 0 or 1'); end
if cf<0, error('CF must be positive'); 
elseif abs(cf-2*pi)>pi/4, warning('Does CF make sense?'); end
if ki<4, error('KI must be >= 4'); end
if rem(sr,1)|(sr<1), error('SR must be integer >= 1'); end
if abs(cj)~=1, error('CJ must be 1 or -1'); end
if abs(sc*sr)<1, error('abs(SC*SR) must be >= 1');
elseif abs(sc*sr)<2, warning('abs(SC*SR) should be >= 2 for data analysis'); end
if abs(ot)>floor(abs(sc*sr)*ki), error('abs(OT) must be <= floor(abs(SC*SR)*KI)'); end
% else disp('skipped')
end	% skip all checks if 10th argument is 'skip'

% Begin evaluation...
scsr=sc*sr;			% sample scale
fsk=floor(abs(scsr)*ki);	% wavelet width
tprime=[-fsk:fsk];		% sample span
ltp=length(tprime);		% = 2*fsk+1
lpsi=ltp;			% full wavelet duration
if nt,lpsi=nt; end		% truncated wavelet length
flp2=floor(lpsi/2);		% wavelet midpoint
% select appropriate part of wavelet
toff=max(0,fsk-flp2);
if ot<0                         % left edge
    tprime=tprime(1:min(ltp,lpsi))+toff;
elseif ot>0                     % right edge
    tprime=tprime(max(1,ltp-lpsi+1):ltp)-toff;
elseif (lpsi<ltp)&(mod(lpsi,2)) % ot=0 only for odd lpsi<ltp
    tprime=tprime(toff+[1:lpsi]); 
elseif (lpsi<ltp)&(mod(lpsi-1,2)) % should only be possible thru pilot error
    error('OT=0 for even lpsi<ltp')
end
eta=(tprime-ot)/scsr;			% argument
theta=exp(cj*i*cf*eta);			% wave
phi=exp(-eta.^2/2);         		% window
if aa, aa=sum(theta.*phi)/sum(phi); end	% adaptive admissibility term
cs=sqrt(2)*(pi*scsr^6).^(-1/4); 	% symmetric normalization
psi=cs*phi.*(theta-aa);			% raw wavelet
if rn, psi=psi/sqrt(psi*psi'/2*scsr^2); end % renormalize
