function ircwt=ircwtm(rcwt,sr,sax,ki,cf,aa,rn,skip) 
% 
% IRCWTM -- inverse continuous wavelet transform with Morlet basis.
%     Assumes RCWT has unit sample rate and positive scale axis.
%
%     IRCWT = IRCWTM(RCWT) 
%     calculates the inverse continuous wavelet transform of RCWT
%     from scale 2 to scale 100 at 100 scales logarithmically spaced.
%
%     IRCWT = IRCWTM(RCWT,SR) 
%     calculates the inverse continuous wavelet transform of RCWT 
%     where integer SR>=1 defines the unit period for the scale axis in
%     terms of the sample rate.
%
%     IRCWT = IRCWTM(RCWT,SR,SAX) 
%     calculates the inverse continuous wavelet transform of RCWT
%     using scale axis SAX.
%
%     IRCWT = IRCWTM(RCWT,SR,SAX,KI) 
%     calculates the inverse continuous wavelet transform of RCWT 
%     with mother wavelet length 2*floor(SR*KI)+1 for KI>=4. Default KI=6.
%
%     IRCWT = IRCWTM(RCWT,SR,SAX,KI,CF) 
%     calculates the inverse continuous wavelet transform of RCWT 
%     with mother wavelet central frequency CF ~ 2*PI+1/4/PI.
%
%     IRCWT = IRCWTM(RCWT,SR,SAX,KI,CF,AA) 
%     calculates the inverse continuous wavelet transform of RCWT 
%     enforcing admissibility for AA=1. Default AA=0 neglects admissibility.
%
%     IRCWT = IRCWTM(RCWT,SR,SAX,KI,CF,AA,RN) 
%     calculates the inverse continuous wavelet transform without 
%     renormalization for RN=0. Default RN=1 enforces renormalization.
%
%     IRCWT = IRCWTM(RCWT,SR,SAX,KI,CF,AA,RN,SKIP) 
%     skips all sanity checks for SKIP='skip' -- not recommended.
%
%     NOTE: Any input parameter may be replaced by [] to accept default
%     value. Eg, IRCWTM(RCWT,[],SAX) calculates the IRCWT using scale axis
%     SAX with unit sample rate.
%
%     See also: EDGEMORLET, RCWTM, XYCWTM, SHOWRCWT, AWDEMO
%
%     AlphaWavelet Toolbox Version 1.0.1
%     Copyright (C) 2010 Alphawave Research, robjohnson@alphawaveresearch.com
%     Released under GNU GPLv3 -- see Contents.m for more info.
%

%     This file is part of AlphaWavelet.
%
%     AlphaWavelet is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or (at
%     your option) any later version.
%
%     AlphaWavelet is distributed in the hope that it will be useful, but
%     WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%     General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see `http://www.gnu.org/licenses/'.
%
%     **************************************************************************

% Preliminaries...
if nargin>8, error('nargin > 8, try help ircwtm'); end
if ~((nargin>7)&&(strcmpi(skip,'skip')))	% skip all checks if 8th argument is 'skip'
% Initialization...	
if nargin<1, error('IRCWT must be present'); end 	% must enter a signal
if nargin<2 || isempty(sr), sr=1; end		% default sample rate
if nargin<3 || isempty(sax), sax=logspace(log10(2),log10(100),100); end	% default scale axis
if nargin<4 || isempty(ki), ki=6; end		% default resolution width
if nargin<5 || isempty(cf), cf=2*pi+1/4/pi; end	% default central frequency
if nargin<6 || isempty(aa), aa=0; end		% default admissibility off
if nargin<7 || isempty(rn), rn=1; end		% default renormalization on
% Sanity check...
[slen,lsig]=size(rcwt);		% get rcwt dimensions
if slen~=length(sax), error('IRCWT dimensions do not match SAX'); end
if rem(sr,1)|(sr<1), error('SR must be integer >= 1'); end
if ~((rn==1)|(rn==0)), error('RN must be 1 or 0'); end
if ~((aa==1)|(aa==0)), error('AA must be 0 or 1'); end
if cf<0, error('CF must be positive'); 
elseif abs(cf-2*pi)>pi/4, warning('Does CF make sense?'); end
if ki<4, error('KI must be >= 4'); end
skip='skip'; % should be good from here
% else disp('skipped')
end	% skip all checks if 8th argument is 'skip'

% Begin evaluation...
ircwt=zeros(1,lsig);   	% initialize memory
irc=ones(slen,lsig);	% initialize memory
fls2=floor(lsig/2);	% signal midpoint
for ksc=1:slen   	% for sc
    sck=sax(ksc);
    fks=floor(ki*sck*sr);
    koff=max(0,fks-fls2);
    bflag=1;    
    for kt=1:lsig 	% for t
        if kt<=fls2
            shft=min([0 kt-fks-1])+koff;
            ktmin=max([1 kt-fks]);
            ktmax=min([min([1+2*fks kt+2*fks])+(ktmin-1) lsig]);
        else
            shft=max([0 kt+fks-lsig])-koff;
            ktmin=max([min([lsig-2*fks kt-fks]) 1]);
            ktmax=min([lsig kt+fks]);
        end
        if shft | bflag, bflag=shft;    % only compute b=0 wavelet once
            psi=edgemorlet(sck,-1,sr,ki,cf,aa,rn,shft,lsig,skip); % use conjugate 
        end
%        ircwt(kt)=ircwt(kt)+real(psi*rcwt(ks,ktmin:ktmax).')*sam(ks);   % rect quadrature
	irc(ksc,kt)=real(psi*rcwt(ksc,ktmin:ktmax).'); % symmetric normalization
    end
end
ircwt=sr*diff(sax)*(irc(1:slen-1,:)+irc(2:slen,:))/2;	% trapezoidal quadrature
