function [rcwt,sax,tax]=rcwtm(sig,sr,smin,smax,slen,slog,ki,cf,aa,rn,skip)
%
% RCWTM -- renormalized continuous wavelet transform with Morlet basis.
%     Assumes SIG has unit sample rate and no mean value.
%
%     RCWT = RCWTM(SIG) 
%     calculates the renormalized continuous wavelet transform of SIG
%     from scale 2 to scale 100 at 100 scales logarithmically spaced.
%
%     RCWT = RCWTM(SIG,SR) 
%     calculates the renormalized continuous wavelet transform of SIG 
%     where integer SR>=1 defines the unit period for the scale axis in
%     terms of the sample rate.
%
%     RCWT = RCWTM(SIG,SR,SMIN,SMAX) 
%     calculates the renormalized continuous wavelet transform of SIG
%     from scale SMIN to scale SMAX at 100 scales logarithmically spaced.
%
%     RCWT = RCWTM(SIG,SR,SMIN,SMAX,SLEN) 
%     calculates the renormalized continuous wavelet transform of SIG
%     from scale SMIN to scale SMAX at SLEN scales logarithmically spaced.
%
%     RCWT = RCWTM(SIG,SR,SMIN,SMAX,SLEN,SLOG) 
%     calculates the renormalized continuous wavelet transform of SIG 
%     from scale SMIN to scale SMAX at SLEN scales logarithmically spaced
%     for SLOG=1 or linearly spaced for SLOG=0.
%
%     RCWT = RCWTM(SIG,SR,SMIN,SMAX,SLEN,SLOG,KI) 
%     calculates the renormalized continuous wavelet transform of SIG 
%     with mother wavelet length 2*floor(SR*KI)+1 for KI>=4. Default KI=6.
%
%     RCWT = RCWTM(SIG,SR,SMIN,SMAX,SLEN,SLOG,KI,CF) 
%     calculates the renormalized continuous wavelet transform of SIG 
%     with mother wavelet central frequency CF ~ 2*PI+1/4/PI.
%
%     RCWT = RCWTM(SIG,SR,SMIN,SMAX,SLEN,SLOG,KI,CF,AA) 
%     calculates the renormalized continuous wavelet transform of SIG 
%     enforcing admissibility for AA=1. Default AA=0 neglects admissibility.
%
%     RCWT = RCWTM(SIG,SR,SMIN,SMAX,SLEN,SLOG,KI,CF,AA,RN) 
%     calculates the continuous wavelet transform without renormalization
%     for RN=0. Default RN=1 enforces renormalization.
%
%     RCWT = RCWTM(SIG,SR,SMIN,SMAX,SLEN,SLOG,KI,CF,AA,RN,SKIP) 
%     skips all sanity checks for SKIP='skip' -- not recommended.
%
%     [RCWT,SAX] = RCWTM(...) 
%     returns the renormalized continuous wavelet transform in RCWT and 
%     the scale axis in SAX.
%
%     [RCWT,SAX,TAX] = RCWTM(...) 
%     returns the renormalized continuous wavelet transform in RCWT and 
%     the time axis in sample units in TAX.
%
%     NOTE: Any input parameter may be replaced by [] to accept default
%     value. Eg, RCWTM(SIG,[],[],[],[],0) calculates the RCWT from scale 2 to
%     scale 100 at 100 scales linearly spaced with unit sample rate.
%
%     See also: EDGEMORLET, IRCWTM, XYCWTM, SHOWRCWT, AWDEMO
%
%     AlphaWavelet Toolbox Version 1.0.1
%     Copyright (C) 2010 Alphawave Research, robjohnson@alphawaveresearch.com
%     Released under GNU GPLv3 -- see Contents.m for more info.
%

%     This file is part of AlphaWavelet.
%
%     AlphaWavelet is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or (at
%     your option) any later version.
%
%     AlphaWavelet is distributed in the hope that it will be useful, but
%     WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%     General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see `http://www.gnu.org/licenses/'.
%
%     **************************************************************************

% Preliminaries...
if nargin>11, error('nargin > 11, try help rcwtm'); end
if ~((nargin>10)&&(strcmpi(skip,'skip')))	% skip all checks if 11th argument is 'skip'
% Initialization...	
if nargin<1, error('SIG must be present'); end 	% must enter a signal
if nargin<2 || isempty(sr), sr=1; end		% default sample rate
if nargin<3 || isempty(smin), smin=2; end	% default smin=2
if nargin<4 || isempty(smax), smax=100; end	% default smax=100
if nargin<5 || isempty(slen), slen=100; end	% default slen=100
if nargin<6 || isempty(slog), slog=1; end	% default log spacing
if nargin<7 || isempty(ki), ki=6; end		% default resolution width
if nargin<8 || isempty(cf), cf=2*pi+1/4/pi; end	% default central frequency
if nargin<9 || isempty(aa), aa=0; end		% default admissibility off
if nargin<10 || isempty(rn), rn=1; end		% default renormalization on
% Sanity check...
if length(sig)~=prod(size(sig)), error('SIG must be a vector'); end
if ~((rn==1)|(rn==0)), error('RN must be 1 or 0'); end
if ~((aa==1)|(aa==0)), error('AA must be 0 or 1'); end
if cf<0, error('CF must be positive'); 
elseif abs(cf-2*pi)>pi/4, warning('Does CF make sense?'); end
if ki<4, error('KI must be >= 4'); end
if rem(sr,1)|(sr<1), error('SR must be integer >= 1'); end
if ~((slog==1)|(slog==0)), error('SLOG must be 0 or 1'); end
if rem(slen,1)|(slen<4), error('SLEN must be integer >=4'); end
if smax<smin, error('SMAX must be > SMIN'); end
if smax*sr>2*length(sig), warning('SMAX*SR is > 2*length(SIG)'); end
if smin*sr<2, error('SMIN*SR must be > 2'); end
skip='skip'; % should be good from here
% else disp('skipped')
end	% skip all checks if 11th argument is 'skip'

% Begin evaluation...
sig=sig(:);     	% make signal a column vector
lsig=length(sig);	% signal length
tax=1:lsig;		% signal duration
if slog, sax=logspace(log10(smin),log10(smax),slen);	% log spacing
else sax=linspace(smin,smax,slen); end			% lin spacing
rcwt=i*ones(slen,lsig); % initialize memory
fls2=floor(lsig/2);	% signal midpoint
for ksc=1:slen   	% for sc
    sck=sax(ksc);
    fks=floor(ki*sck*sr);
    koff=max(0,fks-fls2);
    bflag=1;    
    for kt=1:lsig 	% for t
        if kt<=fls2
            shft=min([0 kt-fks-1])+koff;
            ktmin=max([1 kt-fks]);
            ktmax=min([min([1+2*fks kt+2*fks])+(ktmin-1) lsig]);
        else
            shft=max([0 kt+fks-lsig])-koff;
            ktmin=max([min([lsig-2*fks kt-fks]) 1]);
            ktmax=min([lsig kt+fks]);
        end
        if shft | bflag, bflag=shft;	% only compute b=0 wavelet once
            psi=edgemorlet(sck,-1,sr,ki,cf,aa,rn,shft,lsig,skip); % use conjugate 
        end        
        rcwt(ksc,kt)=psi*sig(ktmin:ktmax);	% symmetric normalization
    end
end
