function varargout=showrcwt(rcwt,por,clvl,sr,ki,sax,tax,slim,tlim,stik,ttik,doplot,dofill)
%
% SHOWRCWT -- displays PSD, MWP, IIP, and PHASE for RCWT in current figure.
%     PSD, MWP, and IIP can be either peak (default) or rms normalized.
%     PHASE is defined over the range -pi to pi.
%
%     H = SHOWRCWT(RCWT)
%     displays peak normalized PSD, MWP, IIP, and PHASE for RCWT using
%     default SAX (see RCWTM) returning the figure handle in H.
%
%     H = SHOWRCWT(RCWT,POR)
%     displays POR normalized PSD, MWP, IIP, and PHASE for RCWT using
%     default SAX (see RCWTM) where POR = 'peak' or 'rms'.
%
%     H = SHOWRCWT(RCWT,POR,CLVL)
%     displays POR normalized PSD, MWP, IIP, and PHASE for RCWT using CLVL
%     number of contour levels for PSD. Default CLVL=9.
%
%     H = SHOWRCWT(RCWT,POR,CLVL,SR,KI)
%     displays POR normalized PSD, MWP, IIP, and PHASE for RCWT with mother
%     wavelet length 2*floor(SR*KI)+1 for KI>=4 where SR>=1 defines the unit
%     period for the scale axis in terms of the sample rate. Default KI=6.
%
%     H = SHOWRCWT(RCWT,POR,CLVL,SR,KI,SAX,TAX)
%     displays POR normalized PSD, MWP, IIP, and PHASE for RCWT using scale
%     axis SAX and time axis TAX.
%
%     H = SHOWRCWT(RCWT,POR,CLVL,SR,KI,SAX,TAX,SLIM,TLIM)
%     displays POR normalized PSD, MWP, IIP, and PHASE for RCWT using SLIM
%     and TLIM as the scale and time axis limits.
%
%     H = SHOWRCWT(RCWT,POR,CLVL,SR,KI,SAX,TAX,SLIM,TLIM,STIK,TTIK)
%     displays POR normalized PSD, MWP, IIP, and PHASE for RCWT using STIK
%     and TTIK as the scale and time axis ticks.
%
%     [H,PSD,MWP,IIP,PHASE,SAM,COA,COI] = SHOWRCWT(...)
%     returns POR normalized PSD, MWP, IIP, and PHASE for RCWT. The scale
%     axis measure is returned in SAM, and the cones-of-admissibility and
%     influence are returned in COA and COI.
%
%     [PSD,MWP,IIP,PHASE,SAM,COA,COI] = SHOWRCWT(...,TTIK,DOPLOT)
%     returns POR normalized PSD, MWP, IIP, PHASE, SAM, COA, and COI without
%     producing a plot for DOPLOT='noplot'. Default DOPLOT='doplot'.
%
%     [H,PSD,MWP,IIP,PHASE,SAM,COA,COI] = SHOWRCWT(...,DOPLOT,DOFILL)
%     returns POR normalized PSD, MWP, IIP, PHASE, SAM, COA, and COI with
%     filled contours for DOFILL='dofill'. Default DOFILL='nofill'.
%
%     NOTE: Any input parameter may be replaced by [] to accept default
%     value. Eg, SHOWRCWT(RCWT,[],20) displays the peak normalized PSD for
%     RCWT from scale 2 to scale 100 at 100 scales logarithmically spaced
%     using 20 contour levels.
%
%     See also: EDGEMORLET, RCWTM, IRCWTM, XYCWTM, AWDEMO
%
%     AlphaWavelet Toolbox Version 1.0.1
%     Copyright (C) 2010 Alphawave Research, robjohnson@alphawaveresearch.com
%     Released under GNU GPLv3 -- see Contents.m for more info.
%

%     This file is part of AlphaWavelet.
%
%     AlphaWavelet is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or (at
%     your option) any later version.
%
%     AlphaWavelet is distributed in the hope that it will be useful, but
%     WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%     General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see `http://www.gnu.org/licenses/'.
%
%     **************************************************************************

% Premilinaries...
if nargin<1 || isempty(rcwt), error('RCWT must be present'); end
[slen,lsig]=size(rcwt);		% get rcwt dimensions
if nargin<2 || isempty(por), por='peak'; end	% default peak psd normalization
if ~sum(strcmpi(por,{'peak','rms'})), error('POR must be either ''peak'' or ''rms'''); end
if nargin<3 || isempty(clvl), clvl=9; end	% default number of contour levels
if rem(clvl,1)|(clvl<=0), error('CLVL must be positive integer'); end
if nargin<4 || isempty(sr), sr=1; end		% default scale unit
if rem(sr,1)|(sr<1), error('SR must be integer >= 1'); end
if nargin<5 || isempty(ki), ki=6; end		% default resolution width
if ki<4, error('KI must be >= 4'); end
if nargin<6 || isempty(sax), sax=logspace(log10(2),log10(100),100); end	% default scale axis
if nargin<7 || isempty(tax), tax=1:lsig; end	% get tax from rcwt
if (slen~=length(sax))|(lsig~=length(tax)), 
	error('RCWT dimensions do not match SAX and/or TAX'); end
% no checks on limits or ticks
if nargin<8 || isempty(slim), slim=[sax(1) sax(slen)]; end
if nargin<9 || isempty(tlim), tlim=[tax(1) tax(lsig)]; end
if nargin<10 || isempty(stik), stik=round(sax(round(linspace(1,slen,5)))); end
if nargin<11 || isempty(ttik), ttik=round(tax(round(linspace(1,lsig,5)))); end
if nargin<12 || isempty(doplot), doplot='doplot'; end	% default produce figure
if ~sum(strcmpi(doplot,{'doplot','noplot'})), 
	error('DOPLOT must be either ''doplot'' or ''noplot'''); end
if nargin<13 || isempty(dofill), dofill='nofill'; end	% default no filled contours
if ~sum(strcmpi(dofill,{'dofill','nofill'})), 
	error('DOFILL must be either ''dofill'' or ''nofill'''); end
if nargin>13, error('nargin > 13, try help showrcwt'); end

% Begin evaluation...
psd=abs(rcwt).^2;	% start with rms psd
if strcmpi(por,'peak'), psd=2*psd; end	% make peak psd
mwp=mean(psd,2);	% mean wavelet power
sam=diff(sax); sam(slen)=sam(slen-1)^2/sam(slen-2);	% get scale axis measure
if diff(sam(2:3))==0, yscl='lin'; else yscl='log'; end	% linear or log scale axis
%iip=sr*sam*psd;	% rectangular quadrature	% integrated instant power
iip=sr*sam(1:slen-1)*(psd(1:slen-1,:)+psd(2:slen,:))/2;	% trapezoidal quadrature
phase=atan2(imag(rcwt),real(rcwt));	% get wavelet phase
tndx=1:lsig; mtndx=(1+lsig)/2; 		% get tax index
coa=(mtndx-1-abs(mtndx-tndx))/ki; coa=max(coa,2)/sr;  	% get cone-of-admissibility
coi=(mtndx-1-abs(mtndx-tndx))/sqrt(2); coi=max(coi,2)/sr; % get cone-of-influence

if strcmpi(doplot,'doplot')	% Do plot
dp=1; h=gcf; clf, df=strcmpi(dofill,'dofill');
subplot(2,2,1), if df, contourf(tax,sax,log10(psd),clvl), else contour(tax,sax,log10(psd),clvl), end
set(gca,'ydir','reverse','yscale',yscl,'ylim',slim,'ytick',stik,'xlim',tlim,'xtick',ttik), hold on, 
plot(tax,coa,'k-',tax,coi,'k-'), % plot coa and coi
if lsig/sr<slim(2), plot(tlim,lsig/sr*[1 1],'k-'); end, hold off, % plot duration scale
subplot(2,2,2), semilogx(mwp,sax), xl=xlim; lxl=[floor(log10(xl(1))) ceil(log10(xl(2)))]; 
if mod(diff(lxl),2), lxl(1)=lxl(1)-1; end, xtik=10.^linspace(lxl(1),lxl(2),min(factor(diff(lxl))+1));
set(gca,'ydir','reverse','yscale',yscl,'ylim',slim,'ytick',stik,'xlim',10.^lxl,'xtick',xtik)
if lsig/sr<slim(2), hold on, plot(xlim,lsig/sr*[1 1],'k-'); hold off, end, % plot duration scale
subplot(2,2,3), plot(tax,iip), mi2=max(max(iip),2*mean(iip)); flmi2=10^floor(log10(mi2)); cmi2=ceil(mi2/flmi2);
set(gca,'xlim',tlim,'xtick',ttik,'ylim',[0 cmi2*flmi2],'ytick',linspace(0,cmi2*flmi2,min(factor(cmi2))+1)), 
subplot(2,2,4), if df, [cfc,cfh]=contourf(tax,sax,phase,pi*linspace(-1,1,5)); set(cfh,'edgecolor','none'), 
	else contour(tax,sax,phase,pi*linspace(-1,1,5)), end
set(gca,'ydir','reverse','yscale',yscl,'ylim',slim,'ytick',stik,'xlim',tlim,'xtick',ttik), hold on, 
plot(tax,coa,'k-',tax,coi,'k-'), % plot coa and coi
if lsig/sr<slim(2), plot(tlim,lsig/sr*[1 1],'k-'); end, hold off, % plot duration scale
varargout{1}=h; 
else dp=0; end			% Do no plot

if nargout>0+dp, varargout{1+dp}=psd; end
if nargout>1+dp, varargout{2+dp}=mwp; end
if nargout>2+dp, varargout{3+dp}=iip; end
if nargout>3+dp, varargout{4+dp}=phase; end
if nargout>4+dp, varargout{5+dp}=sam; end
if nargout>5+dp, varargout{6+dp}=coa; end
if nargout>6+dp, varargout{7+dp}=coi; end
if nargout>7+dp, error('Too many output arguments'); end
