function varargout=xycwtm(rcwt1,rcwt2,sax1,sax2,sr,tax1,tax2,tlag)
%
% XYCWTM -- wavelet power spectral density cross correlation with lag.
%     Requires RCWT1 and RCWT2 to have equivalent sample rates and 
%     overlapping scale and time axis values.
%     
%     XCWT = XYCWTM(RCWT1,RCWT2)
%     calculates the normalized cross correlation of the wavelet power
%     spectral densities RCWT1 and RCWT2 using default scale and time axes.
%     
%     XCWT = XYCWTM(RCWT1,RCWT2,SAX1)
%     calculates the normalized cross correlation of the wavelet power
%     spectral densities RCWT1 and RCWT2 using scale axis SAX1=SAX2.
%     
%     XCWT = XYCWTM(RCWT1,RCWT2,SAX1,SAX2)
%     calculates the normalized cross correlation of the wavelet power
%     spectral densities RCWT1 and RCWT2 using scale axes SAX1 and SAX2.
%     
%     XCWT = XYCWTM(RCWT1,RCWT2,SAX1,SAX2,SR)
%     calculates the normalized cross correlation of the wavelet power
%     spectral densities RCWT1 and RCWT2 using integer sample rate SR>=1.
%     
%     XCWT = XYCWTM(RCWT1,RCWT2,SAX1,SAX2,SR,TAX1)
%     calculates the normalized cross correlation of the wavelet power
%     spectral densities RCWT1 and RCWT2 using time axis TAX1=TAX2.
%     
%     XCWT = XYCWTM(RCWT1,RCWT2,SAX1,SAX2,SR,TAX1,TAX2)
%     calculates the normalized cross correlation of the wavelet power
%     spectral densities RCWT1 and RCWT2 using time axes TAX1 and TAX2.
%     
%     XCWT = XYCWTM(RCWT1,RCWT2,SAX1,SAX2,SR,TAX1,TAX2,TLAG)
%     calculates the normalized cross correlation of the wavelet power
%     spectral densities RCWT1(SAX,TAX1) and RCWT2(SAX,TAX2-TLAG).
%     Note that TLAG*SR must be an integer.
%     
%     [XCWT,YCWT] = XYCWTM(...)
%     returns the normalized cross correlation of the wavelet power
%     spectral densities in XCWT and the co-correlation in YCWT. Use YCWT
%     to produce the correlated signal reconstruction.
%     
%     [SAXC,TAXC,XCWT] = XYCWTM(...)
%     [SAXC,TAXC,XCWT,YCWT] = XYCWTM(...)
%     return the common scale and time axes in SAXC and TAXC.
%
%     NOTE: Any input parameter may be replaced by [] to accept default
%     value. Eg, XYCWTM(RCWT1,RCWT2,[],[],2) calculates the normalized
%     cross correlation with lag 2 and time axis in sample units determined
%     from the dimensions of RCWT1 assuming a scale axis from 2 to 100 at 
%     100 scales logarithmically spaced and unit sample rate.
%
%     See also: EDGEMORLET, RCWTM, IRCWTM, SHOWRCWT, AWDEMO
%
%     AlphaWavelet Toolbox Version 1.0.1
%     Copyright (C) 2010 Alphawave Research, robjohnson@alphawaveresearch.com
%     Released under GNU GPLv3 -- see Contents.m for more info.
%

%     This file is part of AlphaWavelet.
%
%     AlphaWavelet is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or (at
%     your option) any later version.
%
%     AlphaWavelet is distributed in the hope that it will be useful, but
%     WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%     General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see `http://www.gnu.org/licenses/'.
%
%     **************************************************************************

% Preliminaries...
if isempty(rcwt1)|isempty(rcwt2), error('RCWT1 and RCWT2 must be present'); end
[slen1,lsig1]=size(rcwt1);		% get rcwt1 dimensions
[slen2,lsig2]=size(rcwt2);		% get rcwt2 dimensions
if nargin<3 || isempty(sax1), sax1=logspace(log10(2),log10(100),100); end % default scale axis
if nargin<4 || isempty(sax2), sax2=sax1; end	% default scale axis
if nargin<5 || isempty(sr), sr=1; end		% default scale unit
if nargin<6 || isempty(tax1), tax1=1:lsig1; end	% get tax1 from rcwt1
if nargin<7 || isempty(tax2), tax2=tax1; end	% get tax2 from tax1
if nargin<8 || isempty(tlag), tlag=0; end	% default lag is zero
if nargin>8, error('nargin > 8, try help xycwtm'); end
% Sanity check...
if (slen1~=length(sax1))|(lsig1~=length(tax1)), 
	error('RCWT1 dimensions do not match SAX1 and/or TAX1'); end
if (slen2~=length(sax2))|(lsig2~=length(tax2)), 
	error('RCWT2 dimensions do not match SAX2 and/or TAX2'); end
if rem(sr,1)|(sr<1), error('SR must be integer >= 1'); end
if rem(tlag*sr,1), error('TLAG*SR must be integer'); end
[tndx1,tndx2]=comndx(tax1,tax2-tlag);	% get common time axis values
if isempty(tndx1), error('TAX1 and TAX2-TLAG have no common values'); end
[sndx1,sndx2]=comndx(sax1,sax2);	% get common scale axis values
if isempty(sndx1), error('SAX1 and SAX2 have no common values'); end

% Begin evaluation...
rcwt1=rcwt1(sndx1,tndx1);	% reduce to common region
rcwt2=rcwt2(sndx2,tndx2);	% reduce to common region
mma1=max(max(abs(rcwt1)));	% normalization
mma2=max(max(abs(rcwt2)));	% normalization
xynorm=sr*sum(diff(sax1(sndx1)));	% normalization
xcwt=sqrt(conj(rcwt1).*rcwt2)/sqrt(mma1*mma2)/xynorm;	% cross correlation
% get co-correlation only if requested for reconstruction phase
if (nargout==2)|(nargout==4), ycwt=sqrt(rcwt1).*sqrt(rcwt2)/sqrt(mma1*mma2)/xynorm; end
switch nargout
	case {0,1}, varargout{1}=xcwt;
	case 2, varargout{1}=xcwt; varargout{2}=ycwt;
	case 3, varargout{1}=sax1(sndx1); varargout{2}=tax1(tndx1); varargout{3}=xcwt;
	case 4, varargout{1}=sax1(sndx1); varargout{2}=tax1(tndx1); varargout{3}=xcwt; varargout{4}=ycwt;
	otherwise, error('Too many output arguments'); 
end % switch
